import { inject, Injectable } from '@angular/core';
import { Subject } from 'rxjs';
import { AuthConfig, OAuthService, OAuthStorage } from 'angular-oauth2-oidc';
import { UserService } from './user.service';
import { User } from '../model/User';
import { Router } from '@angular/router';


@Injectable({
  providedIn: 'root',
})
export class AuthService {
  private readonly authConfig: AuthConfig = {
    issuer: 'https://oauth.simonis.lol/application/o/casino-dev/',
    clientId: 'MDqjm1kcWKuZfqHJXjxwAV20i44aT7m4VhhTL3Nm',
    dummyClientSecret: 'GY2F8te6iAVYt1TNAUVLzWZEXb6JoMNp6chbjqaXNq4gS5xTDL54HqBiAlV1jFKarN28LQ7FUsYX4SbwjfEhZhgeoKuBnZKjR9eiu7RawnGgxIK9ffvUfMkjRxnmiGI5',
    scope: 'openid profile email',
    responseType: 'code',
    redirectUri: window.location.origin + '/auth/callback',
    oidc: true,
    requestAccessToken: true,
    strictDiscoveryDocumentValidation: false,
    skipIssuerCheck: true,
    disableAtHashCheck: true,
  };

  private userService: UserService = inject(UserService);
  private oauthService: OAuthService = inject(OAuthService);
  private oauthStorage: OAuthStorage = inject(OAuthStorage);
  private router: Router = inject(Router);

  private isAuthenticated = new Subject<boolean>();
  private user: User | null = null;

  constructor() {
    console.log(1);
    this.oauthService.setStorage(localStorage);
    this.oauthService.configure(this.authConfig);
    this.oauthService.events.subscribe((event) => {
      console.log(2, event.type);
      if (event.type === 'token_received') {
        console.log(3);
        this.oauthStorage.setItem('jwt', this.getAccessToken());
        this.oauthService.loadUserProfile().then((profile) => {
          console.log(4);
          this.fromUserProfile(profile).subscribe((user) => {
            console.log(5);
            this.user = user;
            console.log(user);
            this.router.navigate(['home']);
          });
        });
      }
    });
    this.oauthService.loadDiscoveryDocumentAndTryLogin().then(() => {
      this.isAuthenticated.next(this.oauthService.hasValidAccessToken());
    });
  }

  login() {
    this.oauthService.initLoginFlow();
  }

  logout() {
    this.oauthService.logOut();
    this.isAuthenticated.next(false);
  }

  isLoggedIn() {
    return this.oauthService.hasValidAccessToken();
  }

  private fromUserProfile(profile: object) {
    return this.userService.getOrCreateUser(profile);
  }

  getAccessToken() {
    return this.oauthService.getAccessToken();
  }
}
